package org.artifactory.opentracing;

import java.util.concurrent.Callable;

/**
 * @author Uriah Levy
 */
public class TraceableCallableDecorator<V> implements Callable<V> {
    private final ArtifactoryTracer artifactoryTracer;
    private Callable<V> callable;
    private ServerTracingSpan currentSpan;

    public TraceableCallableDecorator(Callable<V> callable, ArtifactoryTracer artifactoryTracer, boolean inheritSpan) {
        this.callable = callable;
        this.artifactoryTracer = artifactoryTracer;
        this.currentSpan = inheritSpan ? artifactoryTracer.getCurrentThreadActiveSpan() : null;
    }

    @Override
    public V call() throws Exception {
        if (currentSpan != null) {
            // Set the current span as active for the current thread
            try (ServerTracingScope ignored = artifactoryTracer.activateSpan(currentSpan)) {
                return callable.call();
            }
        }
        // Start a new span that ends at the end of this invocation
        try (ServerTracingScope ignored = artifactoryTracer.startSpanOnCurrentThread("Artifactory Async Invocation")) {
            return callable.call();
        }
    }
}
