#!/usr/bin/env bash

set -e

export PROJECT_ROOT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && cd ../../.. && pwd)"
source "${PROJECT_ROOT_DIR}/build/ci/scripts/runSonarScannerGeneric.sh"

SONAR_PROJECT_BASE_DIR="${SONAR_PROJECT_BASE_DIR:-}"
SONAR_PREV_STEP="${SONAR_PREV_STEP:-}"

function runSonarScan() {
    pushd "${PROJECT_ROOT_DIR}/${SONAR_PROJECT_BASE_DIR}" && ls -lsa
    jfrog rt mvnc
    sonar_runScan
    popd > /dev/null
}

function on_start() {

    sonar_restoreCache "${PROJECT_ROOT_DIR}/${SONAR_PROJECT_BASE_DIR}/" || return 1
    SONAR_ADDITIONAL_OPTIONS="${SONAR_ADDITIONAL_OPTIONS} -Dsonar.java.libraries=${HOME}/.m2"

    echo -e "\n\n==> Trying to find classes ." >&2
    find ${PROJECT_ROOT_DIR} -name classes

    echo -e "\n\n==> Trying to find jacoco ." >&2
    find ${PROJECT_ROOT_DIR} -name jacoco.xml
    #popd > /dev/null

    return 0
}

function on_execute() {

    classes_dirs=$(find ${PROJECT_ROOT_DIR}/${SONAR_PROJECT_BASE_DIR} -name classes | wc -l)
    if pipeline_shouldRunSonar && [ $classes_dirs -eq 0 ]; then
        echo "[ERROR] Artifactory class files were not found. This usually means the ${SONAR_PREV_STEP} step was not executed before this step. In order to avoid this error, please make sure to trigger the ${SONAR_PREV_STEP}, which will in turn trigger this step" >&2
        exit 1
    fi
    runSonarScan
}

function on_success() {
    sonar_cleanCache
}

function on_complete() {
    return 0
}
