package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.itest.UniqueJvmForkTestIdentifier;
import org.artifactory.storage.db.upgrades.common.AsyncUpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.storage.DbType;
import org.jfrog.storage.util.PostgresqlUtils;
import org.testng.Assert;
import org.testng.SkipException;
import org.testng.annotations.Test;

import java.sql.SQLException;

/**
* @author Tomer Slutsky
* This class covers the node_props converters v111-v116
*/
public class V116AsyncUpgradePostgresSqlNodePropsIndexesTest extends AsyncUpgradeBaseTest {

    @Test
    public void testNodesPropsPropValueKeyIndex() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            String message = "This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test.";
            logger.info(message);
            throw new SkipException(message);
        }
        String indexDef = PostgresqlUtils.getPostgresqlIndexDefInfo(jdbcHelper, "node_props_prop_value_key_idx")
                .replace(UniqueJvmForkTestIdentifier.get() + ".", "");
        String query =
                "CREATE INDEX node_props_prop_value_key_idx ON node_props USING btree (prop_value, prop_key)";
        assertIndexDef(query, indexDef);
    }

    @Test
    public void testNodesPropsPropKeyValueIndex() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            String message = "This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test.";
            logger.info(message);
            throw new SkipException(message);
        }
        String indexDef = PostgresqlUtils.getPostgresqlIndexDefInfo(jdbcHelper, "node_props_prop_key_value_idx")
                .replace(UniqueJvmForkTestIdentifier.get() + ".", "");
        String query =
                "CREATE INDEX node_props_prop_key_value_idx ON node_props USING btree (prop_key, prop_value)";
        assertIndexDef(query, indexDef);
    }

    @Test
    public void testNodesPropsNodePropValueIndex() throws SQLException {
        if (dbProperties.getDbType() != DbType.POSTGRESQL) {
            String message = "This test is for Postgresql only, found " + dbProperties.getDbType() + ". Skipping test.";
            logger.info(message);
            throw new SkipException(message);
        }
        String indexDef = PostgresqlUtils.getPostgresqlIndexDefInfo(jdbcHelper, "node_props_node_prop_value_idx")
                .replace(UniqueJvmForkTestIdentifier.get() + ".", "");
        String query =
                "CREATE INDEX node_props_node_prop_value_idx ON node_props USING btree (node_id, prop_key, prop_value)";
        assertIndexDef(query, indexDef);
    }

    private void assertIndexDef(String index, String indexDef) {
        Assert.assertEquals(indexDef, index);
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return ArtifactoryDBVersion.v116;
    }
}
