/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.resource.admin.configuration.proxies;

import com.google.common.annotations.VisibleForTesting;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.model.proxies.ProxiesModel;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.rest.common.service.RestService;
import org.artifactory.rest.exception.AuthorizationRestException;
import org.artifactory.ui.rest.model.admin.configuration.proxy.Proxy;
import org.artifactory.ui.rest.service.admin.configuration.ConfigServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * @author Chen Keinan
 */
@Path("proxies")
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class ProxyResource extends BaseResource {
    protected ConfigServiceFactory configServiceFactory;
    private AuthorizationService authorizationService;

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response createProxies(Proxy proxy) {
        return runService(configServiceFactory.createProxiesService(), proxy);
    }

    @PUT
    @Path("{id: [^/]+}")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response updateProxies(Proxy proxy) {
        return runService(configServiceFactory.updateProxiesService(), proxy);
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getProxies() {
        if (!authorizationService.isAnyProjectAdmin()) {
            throw new AuthorizationRestException();
        }
        // password should be returned for admin only. Project admin should see username only.
        return runService(configServiceFactory.getProxiesService(), authorizationService.isAdmin());
    }

    @GET
    @Path("{id: [^/]+}")
    @Produces(MediaType.APPLICATION_JSON)
    public Response getProxy() {
        if (!authorizationService.isAnyProjectAdmin()) {
            throw new AuthorizationRestException();
        }
        return runService(configServiceFactory.getProxiesService());
    }

    @Path("deleteProxies")
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({AuthorizationService.ROLE_ADMIN})
    public Response deleteProxies(ProxiesModel proxiesModel) {
        return runService(configServiceFactory.deleteProxiesService(), proxiesModel);
    }

    @GET
    @Path("proxyKeys")
    @Produces(MediaType.APPLICATION_JSON)
    public Response getProxyKeys() {
        if (!authorizationService.isAnyProjectAdmin()) {
            throw new AuthorizationRestException();
        }
        return runService(configServiceFactory.getProxyKeysService());
    }

    @Autowired
    public void setConfigServiceFactory(ConfigServiceFactory configServiceFactory) {
        this.configServiceFactory = configServiceFactory;
    }

    @Autowired
    public void setAuthorizationService(AuthorizationService authorizationService) {
        this.authorizationService = authorizationService;
    }

    @Override
    @VisibleForTesting
    protected <Y> Response runService(RestService<Y> service, Y model) {
        return super.runService(service, model);
    }
}
