package org.artifactory.rest.resource.system;

import org.artifactory.api.mail.MailService;
import org.artifactory.api.rest.mail.MailMessageRequest;
import org.artifactory.rest.resource.email.EmailResource;
import org.artifactory.util.EmailException;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.servlet.http.HttpServletResponse;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.testng.AssertJUnit.assertEquals;

class MailResourceTest {

    @Mock
    private MailService mailService;

    private AutoCloseable autoCloseable;

    @BeforeEach
    public void beforeMethod() throws Exception {
        autoCloseable = MockitoAnnotations.openMocks(this);
    }

    @Test
    void testSendMail() {
        EmailResource mailResource = new EmailResource(mailService);
        doNothing().when(mailService).sendMail(any(), eq("subject"), any());
        MailMessageRequest mailReq = new MailMessageRequest("a@a.com", "subject", "body");
        assertEquals(mailResource.sendMail(mailReq).getStatus(), HttpServletResponse.SC_OK);
        verify(mailService, times(1)).sendMail(new String[] {"a@a.com"}, "subject", "body");
    }

    @Test
    void testSendMailFail() {
        EmailResource mailResource = new EmailResource(mailService);
        doThrow(new EmailException("err")).when(mailService).sendMail(any(), eq("1subject1"), any());
        MailMessageRequest mailReq = new MailMessageRequest("a@a.com", "1subject1", "body");
        assertEquals(HttpServletResponse.SC_BAD_REQUEST, mailResource.sendMail(mailReq).getStatus());
        verify(mailService, times(1)).sendMail(new String[] {"a@a.com"}, "1subject1", "body");
    }

    @AfterEach
    void tearDown() throws Exception {
        if (autoCloseable != null) {
            autoCloseable.close();
            autoCloseable = null;
        }
    }
}